@@ -81,13 +81,12 @@ emit_client_error (GdaClient *client, GdaConnection *cnc, const gchar *format, .
 	vsprintf (sz, format, args);
 	va_end (args);
 
-	g_print ("Error: %s\n", sz);
-
 	/* create the error list */
 	error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
 	gda_connection_event_set_description (error, sz);
 	gda_connection_event_set_source (error, "[GDA client]");
-	gda_connection_add_event (cnc, error);
+	if (cnc)
+		gda_connection_add_event (cnc, error);
 }
 
 static void
@@ -471,7 +470,7 @@ gda_client_open_connection (GdaClient *client,
 /**
  * gda_client_open_connection_from_string
  * @client: a #GdaClient object.
- * @provider_id: provider ID to connect to.
+ * @provider_id: provider ID to connect to, or %NULL
  * @cnc_string: connection string.
  * @username: user name.
  * @password: password for @username.
@@ -496,6 +495,10 @@ gda_client_open_connection (GdaClient *client,
  *  <listitem>the USERNAME= and PASSWORD= parts of the @cnc_string</listitem>
  * </itemizedlist>
  *
+ * Additionnally, it is possible to have the connection string
+ * respect the "&lt;provider_name&gt;://&lt;real cnc string&gt;" format, in which case the provider name
+ * and the real connection string will be extracted from that string (note that if @provider_id
+ * is not %NULL then it will still be used as the provider ID).
  *
  * Returns: the opened connection if successful, %NULL if there is
  * an error.
@@ -512,10 +515,28 @@ gda_client_open_connection_from_string (GdaClient *client,
 	LoadedProvider *prv;
 	GdaConnection *cnc = NULL;
 	GList *l;
+	gchar *ptr, *dup;
 
 	g_return_val_if_fail (GDA_IS_CLIENT (client), NULL);
-	g_return_val_if_fail (provider_id != NULL, NULL);
+	g_return_val_if_fail (cnc_string, NULL);
+
+	/* try to see if connection string has the "<provider>://<real cnc string>" format */
+	dup = g_strdup (cnc_string);
+	for (ptr = dup; *ptr; ptr++) {
+		if ((*ptr == ':') && (*(ptr+1) == '/') && (*(ptr+2) == '/')) {
+			if (!provider_id)
+				provider_id = dup;
+			*ptr = 0;
+			cnc_string = ptr + 3;
+		}
+	}
 	
+	if (!provider_id) {
+		g_set_error (error, GDA_CLIENT_ERROR, 0, _("No provider specified"));
+		g_free (dup);
+		return NULL;
+	}
+
 	if (! (options & GDA_CONNECTION_OPTIONS_DONT_SHARE)) {
 		for (l = client->priv->connections; l != NULL; l = l->next) {
 			const gchar *tmp_prov, *tmp_cnc_string;
@@ -524,9 +545,11 @@ gda_client_open_connection_from_string (GdaClient *client,
 			tmp_prov = gda_connection_get_provider (cnc);
 			tmp_cnc_string = gda_connection_get_cnc_string (cnc);
 	
-			if (strcmp(provider_id, tmp_prov) == 0
-			    && (cnc_string != NULL && strcmp (cnc_string, tmp_cnc_string) == 0))
+			if (strcmp (provider_id, tmp_prov) == 0
+			    && (! strcmp (cnc_string, tmp_cnc_string))) {
+				g_free (dup);
 				return cnc;
+			}
 		}
 	}
 
@@ -555,6 +578,8 @@ gda_client_open_connection_from_string (GdaClient *client,
 			     _("Datasource configuration error: no provider specified"));
 	}
 
+	g_free (dup);
+
 	return cnc;
 }
 
