@@ -32,6 +32,7 @@
 #include <libgda/gda-parameter-list.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
+#include <libgda/binreloc/gda-binreloc.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <sys/stat.h>
@@ -42,12 +43,16 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #endif
-#ifdef LIBGDA_WIN32
+#ifdef G_OS_WIN32
 #include <io.h>
 #endif
 
 #define GDA_CONFIG_SECTION_DATASOURCES "/apps/libgda/Datasources"
+#ifdef G_OS_WIN32
 #define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
+#else
+#define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
+#endif
 #define LIBGDA_USER_CONFIG_FILE LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S "config"
 
 /* FIXME: many functions are not thread safe! */
@@ -69,6 +74,10 @@ typedef struct {
 	GList *user;   /* list of GdaConfigSection */
 } GdaConfigClient;
 
+static GStaticRecMutex gda_mutex = G_STATIC_REC_MUTEX_INIT;
+#define GDA_CONFIG_LOCK() g_static_rec_mutex_lock(&gda_mutex)
+#define GDA_CONFIG_UNLOCK() g_static_rec_mutex_unlock(&gda_mutex)
+
 static GdaConfigClient *config_client = NULL;
 static gboolean         can_modify_global_conf; /* TRUE if the current user can modify the system wide config file */
 static gboolean         lock_write_notify = FALSE;
@@ -103,6 +112,7 @@ static void           fam_unlock_notify ();
 static void
 dump_config_client ()
 {
+	GDA_CONFIG_LOCK ();
 	if (config_client) {
 		GList *list;
 
@@ -117,6 +127,7 @@ dump_config_client ()
 			list = list->next;
 		}
 	}
+	GDA_CONFIG_UNLOCK ();
 }
 #endif
 
@@ -268,18 +279,19 @@ get_config_client ()
 	 *	  wait until the operation finishes
 	 */
 	if (! config_client) {
+		GDA_CONFIG_LOCK ();
 		guint len = 0;
 		gchar *full_file = NULL;
 		gchar *user_config = NULL;
 		gboolean has_user_config = FALSE;
 		gchar *memonly;
-		
-		
+		gchar *fname;
+				
 		config_client = g_new0 (GdaConfigClient, 1);
 		xmlKeepBlanksDefault(0);
 
 		/* check if DSN list should only be stored in memory */
-		memonly = g_getenv ("GDA_DSN_LIST_IN_MEMORY");
+		memonly = getenv ("GDA_DSN_LIST_IN_MEMORY");
 		if (memonly) {
 			gsize length;
 			gchar *init_contents;
@@ -290,18 +302,21 @@ get_config_client ()
 				config_client->user = gda_config_parse_config_file (init_contents, length);
 				g_free (init_contents);
 			}
+			GDA_CONFIG_UNLOCK ();
 			return config_client;
 		}
 
+		/* global config file name */
+		fname = gda_gbr_get_file_path (GDA_ETC_DIR, LIBGDA_ABI_NAME, "config", NULL);
+
 		has_user_config = g_get_home_dir () ? TRUE : FALSE;
 		if (has_user_config)
-			user_config = g_strdup_printf ("%s%s", g_get_home_dir (), LIBGDA_USER_CONFIG_FILE);
+			user_config = g_strdup_printf ("%s%s", g_get_home_dir (), LIBGDA_USER_CONFIG_FILE); /* VMA */
 
 		{
 			/* compute system wide rights */
 			FILE *file;
-
-			file = fopen (LIBGDA_GLOBAL_CONFIG_FILE, "a");
+			file = fopen (fname, "a");
 			if (file) {
 				can_modify_global_conf = TRUE;
 #ifdef GDA_DEBUG_NO
@@ -339,11 +354,10 @@ get_config_client ()
 							       fam_callback, NULL);
 				
 				fam_conf_global = g_new0 (FAMRequest, 1);
-				res = FAMMonitorFile (fam_connection, LIBGDA_GLOBAL_CONFIG_FILE, fam_conf_global, 
+				res = FAMMonitorFile (fam_connection, fname, fam_conf_global, 
 						      GINT_TO_POINTER (TRUE));
 #ifdef GDA_DEBUG_NO
-				g_print ("Monitoring changes on file %s: %s\n", 
-					 LIBGDA_GLOBAL_CONFIG_FILE, res ? "ERROR" : "Ok");
+				g_print ("Monitoring changes on file %s: %s\n", fname, res ? "ERROR" : "Ok");
 #endif
 				
 				if (has_user_config) {
@@ -363,8 +377,7 @@ get_config_client ()
 		/*
 		 * load system wide config
 		 */
-		if (g_file_get_contents (LIBGDA_GLOBAL_CONFIG_FILE, &full_file,
-					 &len, NULL)){
+		if (g_file_get_contents (fname, &full_file, &len, NULL)){
 			GList *list;
 			config_client->global = gda_config_parse_config_file (full_file, len);
 			g_free (full_file);
@@ -377,9 +390,12 @@ get_config_client ()
 			}
 		}
 
-		if (! has_user_config) 
+		if (! has_user_config) {
 			/* this can occur on win98, and maybe other win32 platforms */
+			g_free (fname);
+			GDA_CONFIG_UNLOCK ();
 			return config_client;
+		}
 
 		/*
 		 * load user specific config
@@ -396,7 +412,7 @@ get_config_client ()
 
 				dirpath = g_strdup_printf ("%s%s", g_get_home_dir (), LIBGDA_USER_CONFIG_DIR);
 				if (!g_file_test (dirpath, G_FILE_TEST_IS_DIR)){
-#ifdef LIBGDA_WIN32
+#ifdef G_OS_WIN32
 					if (mkdir (dirpath))
 #else
 					if (mkdir (dirpath, 0700))
@@ -414,14 +430,14 @@ get_config_client ()
 "<?xml version=\"1.0\"?>\n" \
 "<libgda-config>\n" \
 "    <section path=\"/apps/libgda/Datasources/SalesTest\">\n" \
-"        <entry name=\"DSN\" type=\"string\" value=\"URI=%s/sales_test.db\"/>\n" \
+"        <entry name=\"DSN\" type=\"string\" value=\"DB_DIR=%s;DB_NAME=sales_test\"/>\n" \
 "        <entry name=\"Description\" type=\"string\" value=\"Test database for a sales department\"/>\n" \
 "        <entry name=\"Password\" type=\"string\" value=\"\"/>\n" \
 "        <entry name=\"Provider\" type=\"string\" value=\"SQLite\"/>\n" \
 "        <entry name=\"Username\" type=\"string\" value=\"\"/>\n" \
 "    </section>\n" \
 "</libgda-config>\n"
-					str = g_build_filename (LIBGDA_GLOBAL_CONFIG_DIR, DB_FILE, NULL);
+					str = gda_gbr_get_file_path (GDA_ETC_DIR, LIBGDA_ABI_NAME, DB_FILE, NULL);
 					if (g_file_get_contents (str, &full_file, &len, NULL)) {
 						gboolean allok = TRUE;
 						FILE *db;
@@ -459,9 +475,11 @@ get_config_client ()
 				g_warning ("Config file is not readable.");
 		}
 		g_free (user_config);
+		g_free (fname);
 #ifdef GDA_DEBUG_NO
 		dump_config_client ();
 #endif
+		GDA_CONFIG_UNLOCK ();
 	}
 
 	return config_client;
@@ -475,6 +493,7 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
 {
         gboolean res = TRUE;
 
+	GDA_CONFIG_LOCK ();
 	while (fam_connection && FAMPending (fam_connection)) {
                 FAMEvent ev;
 		gboolean is_global;
@@ -485,6 +504,7 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
                         g_source_remove (fam_watch_id);
                         fam_watch_id = 0;
                         fam_connection = NULL;
+			GDA_CONFIG_UNLOCK ();
                         return FALSE;
                 }
 
@@ -512,11 +532,13 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
 #ifdef GDA_DEBUG_NO
 			g_print ("Reloading config files (%s config has changed)\n", is_global ? "global" : "user");
 #endif
+			GDA_CONFIG_LOCK ();
 			gda_config_client_reset ();
 			g_free (config_client);
 			config_client = NULL;
 			/* config_client will be re-created next time a gda_config* call is made */
 			do_notify (NULL);
+			GDA_CONFIG_UNLOCK ();
 			break;
 		case FAMAcknowledge:
 		case FAMStartExecuting:
@@ -529,6 +551,7 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
                 }
 	}
 	
+	GDA_CONFIG_UNLOCK ();
         return res;
 }
 
@@ -616,8 +639,10 @@ gda_config_add_section (const gchar *section_path)
 	section->path = g_strdup (section_path);
 	section->entries = NULL;
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	cfg_client->user = g_list_append (cfg_client->user, section);
+	GDA_CONFIG_UNLOCK ();
 
 	return section;
 }
@@ -632,6 +657,7 @@ gda_config_add_entry (const gchar *section_path,
 	GdaConfigSection *section;
 	GdaConfigClient *cfg_client;
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = g_new0 (GdaConfigEntry, 1);
 	entry->name = g_strdup (name);
@@ -643,6 +669,7 @@ gda_config_add_entry (const gchar *section_path,
 		section = gda_config_add_section (section_path);
 
 	section->entries = g_list_append (section->entries, entry);
+	GDA_CONFIG_UNLOCK ();
 }
 
 static void
@@ -673,6 +700,7 @@ gda_config_client_reset ()
 	if (config_client == NULL)
 		return;
 
+	GDA_CONFIG_LOCK ();
 	if (config_client->global) {
 		for (list = config_client->global; list; list = g_list_next (list)) 
 			free_section ((GdaConfigSection *)(list->data));
@@ -686,6 +714,7 @@ gda_config_client_reset ()
 		g_list_free (config_client->user);
 		config_client->user = NULL;
 	}
+	GDA_CONFIG_UNLOCK ();
 }
 
 static void
@@ -722,12 +751,17 @@ write_config_file ()
 	GdaConfigEntry *entry;
 	gchar *user_config;
 
-	if (lock_write_notify)
+	GDA_CONFIG_LOCK ();
+	if (lock_write_notify) {
+		GDA_CONFIG_UNLOCK ();
 		return;
+	}
 
-	if (dsn_list_only_in_mem)
+	if (dsn_list_only_in_mem) {
 		/* nothing to do */
+		GDA_CONFIG_UNLOCK ();
 		return;
+	}
 
 	/* user specific data sources */
 	cfg_client = get_config_client ();
@@ -784,15 +818,22 @@ write_config_file ()
 		}
 		
 #ifdef HAVE_FAM
-	fam_lock_notify ();
+		fam_lock_notify ();
 #endif
-		if (xmlSaveFormatFile (LIBGDA_GLOBAL_CONFIG_FILE, doc, TRUE) == -1)
+		/* global config file name */
+		gchar *fname;
+		fname = gda_gbr_get_file_path (GDA_ETC_DIR, LIBGDA_ABI_NAME, "config", NULL);
+
+		if (xmlSaveFormatFile (fname, doc, TRUE) == -1)
 			g_warning ("Error saving config data to '%s'", user_config);
+		g_free (fname);
 #ifdef HAVE_FAM
-	fam_unlock_notify ();
+		fam_unlock_notify ();
 #endif
 		xmlFreeDoc (doc);
 	}
+
+	GDA_CONFIG_UNLOCK ();
 }
 
 /*
@@ -813,16 +854,20 @@ gda_config_get_string (const gchar *path)
 {
 	GdaConfigClient *cfg_client;
 	GdaConfigEntry *entry;
+	gchar *retval;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 
 	entry = gda_config_search_entry (cfg_client->user, path, "string");
 	if (!entry)
 		entry = gda_config_search_entry (cfg_client->global, path, "string");
 
-        return (entry && entry->value) ? g_strdup (entry->value) : NULL;
+	retval = (entry && entry->value) ? g_strdup (entry->value) : NULL;
+	GDA_CONFIG_UNLOCK ();
+        return retval;
 }
 
 /**
@@ -838,17 +883,20 @@ gda_config_get_int (const gchar *path)
 {
 	GdaConfigClient *cfg_client;
 	GdaConfigEntry *entry;
+	gint retval;
 
 	g_return_val_if_fail (path != NULL, 0);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 
 	entry = gda_config_search_entry (cfg_client->user, path, "long");
 	if (entry == NULL)
 		entry = gda_config_search_entry (cfg_client->global, path, "long");
+        retval = (entry != NULL && entry->value != NULL) ? atoi (entry->value) : 0;
+	GDA_CONFIG_UNLOCK ();
 
-        return (entry != NULL && entry->value != NULL) ? 
-			atoi (entry->value) : 0;
+	return retval;
 }
 
 /**
@@ -864,17 +912,21 @@ gda_config_get_float (const gchar *path)
 {
 	GdaConfigClient *cfg_client;
 	GdaConfigEntry *entry;
+	gdouble retval;
 
 	g_return_val_if_fail (path != NULL, 0.0);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 
 	entry = gda_config_search_entry (cfg_client->user, path, "float");
 	if (entry == NULL)
 		entry = gda_config_search_entry (cfg_client->global, path, "float");
 
-        return (entry != NULL && entry->value != NULL) ? 
-			g_strtod (entry->value, NULL) : 0.0;
+        retval = (entry != NULL && entry->value != NULL) ? g_strtod (entry->value, NULL) : 0.0;
+	GDA_CONFIG_UNLOCK ();
+
+	return retval;
 }
 
 /**
@@ -890,17 +942,21 @@ gda_config_get_boolean (const gchar *path)
 {
 	GdaConfigClient *cfg_client;
 	GdaConfigEntry *entry;
+	gboolean retval;
 
 	g_return_val_if_fail (path != NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 
 	entry = gda_config_search_entry (cfg_client->user, path, "bool");
 	if (entry == NULL)
 		entry = gda_config_search_entry (cfg_client->global, path, "bool");
 
-        return (entry != NULL && entry->value != NULL) ? 
-			!strcmp (entry->value, "true") : FALSE;
+        retval = (entry != NULL && entry->value != NULL) ? !strcmp (entry->value, "true") : FALSE;
+	GDA_CONFIG_UNLOCK ();
+
+	return retval;
 }
 
 /**
@@ -924,17 +980,21 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 	g_return_val_if_fail (path != NULL, FALSE);
 	g_return_val_if_fail (new_value != NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, "string");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "string");
-		if (entry && !can_modify_global_conf)
+		if (entry && !can_modify_global_conf) {
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
+		}
 	}
 	if (entry == NULL){
 		ptr_last_dash = strrchr (path, '/');
 		if (ptr_last_dash == NULL){
-			g_warning ("%s does not containt any '/'!?", path);
+			g_warning ("%s does not containt any '/'!?", path); 
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
 		}
 		last_dash = ptr_last_dash - path;
@@ -953,6 +1013,7 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 
 	return TRUE;
 }
@@ -978,17 +1039,21 @@ gda_config_set_int (const gchar *path, gint new_value)
 
 	g_return_val_if_fail (path !=NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, "long");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "long");
-		if (entry && !can_modify_global_conf)
+		if (entry && !can_modify_global_conf) {
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
+		}
 	}
 	if (entry == NULL){
 		ptr_last_dash = strrchr (path, '/');
 		if (ptr_last_dash == NULL){
 			g_warning ("%s does not containt any '/'!?", path);
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
 		}
 		last_dash = ptr_last_dash - path;
@@ -1008,6 +1073,7 @@ gda_config_set_int (const gchar *path, gint new_value)
 
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 
 	return TRUE;
 }
@@ -1033,17 +1099,21 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 
 	g_return_val_if_fail (path !=NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, "float");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "float");
-		if (entry && !can_modify_global_conf)
+		if (entry && !can_modify_global_conf) {
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
+		}
 	}
 	if (entry == NULL){
 		ptr_last_dash = strrchr (path, '/');
 		if (ptr_last_dash == NULL){
 			g_warning ("%s does not containt any '/'!?", path);
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
 		}
 		last_dash = ptr_last_dash - path;
@@ -1063,6 +1133,7 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 
 	return TRUE;
 }
@@ -1089,17 +1160,21 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 	g_return_val_if_fail (path !=NULL, FALSE);
 
 	new_value = new_value != 0 ? TRUE : FALSE;
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, "bool");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "bool");
-		if (entry && !can_modify_global_conf)
+		if (entry && !can_modify_global_conf) {
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
+		}
 	}
 	if (entry == NULL){
 		ptr_last_dash = strrchr (path, '/');
 		if (ptr_last_dash == NULL){
 			g_warning ("%s does not containt any '/'!?", path);
+			GDA_CONFIG_UNLOCK ();
 			return FALSE;
 		}
 		last_dash = ptr_last_dash - path;
@@ -1118,6 +1193,7 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 
 	return TRUE;
 }
@@ -1136,12 +1212,14 @@ gda_config_remove_section (const gchar *path)
 
 	g_return_if_fail (path != NULL);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, path);
 	if (!section)
 		section = gda_config_search_section (cfg_client->global, path);
 	if (section == NULL) {
 		g_warning ("Section %s not found!", path);
+		GDA_CONFIG_UNLOCK ();
 		return;
 	}
 
@@ -1150,6 +1228,7 @@ gda_config_remove_section (const gchar *path)
 	free_section (section);
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 }
 
 /**
@@ -1180,12 +1259,14 @@ gda_config_remove_key (const gchar *path)
 	section_path = g_strdup (path);
 	section_path [last_dash] = '\0';
 	entry = NULL;
+	GDA_CONFIG_UNLOCK ();
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, section_path);
 	if (!section)
 		section = gda_config_search_section (cfg_client->global, section_path);
 	if (section == NULL) {
 		g_free (section_path);
+		GDA_CONFIG_UNLOCK ();
 		return;
 	}
 
@@ -1199,8 +1280,10 @@ gda_config_remove_key (const gchar *path)
 
 	g_free (section_path);
 
-	if (entry == NULL)
+	if (entry == NULL) {
+		GDA_CONFIG_UNLOCK ();
 		return;
+	}
 
 	section->entries = g_list_remove (section->entries, entry);
 	free_entry (entry, NULL);
@@ -1211,6 +1294,7 @@ gda_config_remove_key (const gchar *path)
 
 	write_config_file ();
 	do_notify (path);
+	GDA_CONFIG_UNLOCK ();
 }
 
 /**
@@ -1227,15 +1311,19 @@ gda_config_has_section (const gchar *path)
 {
 	GdaConfigSection *section;
 	GdaConfigClient *cfg_client;
+	gboolean retval;
 
 	g_return_val_if_fail (path != NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, path);
 	if (section == NULL)
 		section = gda_config_search_section (cfg_client->global, path);
 
-	return (section != NULL) ? TRUE : FALSE;
+	retval = (section != NULL) ? TRUE : FALSE;
+	GDA_CONFIG_UNLOCK ();
+	return retval;
 }
 
 /**
@@ -1251,9 +1339,11 @@ gda_config_has_key (const gchar *path)
 {
 	GdaConfigEntry *entry;
 	GdaConfigClient *cfg_client;
+	gboolean retval;
 
 	g_return_val_if_fail (path != NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, NULL);
 	if (entry == NULL)
@@ -1261,7 +1351,9 @@ gda_config_has_key (const gchar *path)
 						 path,
 						 NULL);
 
-	return (entry != NULL) ? TRUE : FALSE;
+	retval = (entry != NULL) ? TRUE : FALSE;
+	GDA_CONFIG_UNLOCK ();
+	return retval;
 }
 
 /**
@@ -1269,7 +1361,7 @@ gda_config_has_key (const gchar *path)
  * @path: path to the configuration key.
  *
  * Gets a string representing the type of the value of the given key.
- * The caller is responsible for freeing the returned value.
+ * The caller is responsible of freeing the returned value.
  *
  * Returns: %NULL if not found. Otherwise: "string", "float", "long", "bool".
  */
@@ -1278,20 +1370,22 @@ gda_config_get_type (const gchar *path)
 {
 	GdaConfigEntry *entry;
 	GdaConfigClient *cfg_client;
+	gchar *retval = NULL;
 
 	g_return_val_if_fail (path != NULL, FALSE);
 
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	entry = gda_config_search_entry (cfg_client->user, path, NULL);
 	if (entry == NULL)
 		entry = gda_config_search_entry (cfg_client->global, 
 						 path,
 						 NULL);
-
-	if (entry == NULL)
-		return NULL;
-
-	return g_strdup (entry->type);
+	
+	if (entry)
+		retval = g_strdup (entry->type);
+	GDA_CONFIG_UNLOCK ();
+	return retval;
 }
 
 static GList *
@@ -1306,6 +1400,7 @@ gda_config_list_sections_raw (const gchar *path)
         g_return_val_if_fail (path != NULL, NULL);
 
 	len = strlen (path);
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	for (list = cfg_client->user; list; list = list->next){
 		section = (GdaConfigSection*) list->data;
@@ -1326,7 +1421,7 @@ gda_config_list_sections_raw (const gchar *path)
 				ret = g_list_append (ret, section);
 		}
 	}
-
+	GDA_CONFIG_UNLOCK ();
         return ret;
 }
 
@@ -1382,6 +1477,7 @@ gda_config_list_keys (const gchar * path)
         g_return_val_if_fail (path != NULL, NULL);
 
 	len = strlen (path);
+	GDA_CONFIG_LOCK ();
 	cfg_client = get_config_client ();
 	if (cfg_client->user){
 		for (ls = cfg_client->user; ls; ls = ls->next){
@@ -1412,6 +1508,7 @@ gda_config_list_keys (const gchar * path)
 		}
 	}
 
+	GDA_CONFIG_UNLOCK ();
         return ret;
 }
 
@@ -1559,7 +1656,7 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
  * the system. Each of the nodes in the returned GList
  * is a #GdaProviderInfo.
  *
- * Returns: a GList of #GdaProviderInfo structures. Don't free or modify it!
+ * Returns: a GList of #GdaProviderInfo structures, don't free or modify it!
  */
 GList *
 gda_config_get_provider_list (void)
@@ -1569,11 +1666,15 @@ gda_config_get_provider_list (void)
 	if (!prov_list) {
 		const gchar *from_dir;
 
-		from_dir = g_getenv ("GDA_PROVIDERS_ROOT_DIR");
+		from_dir = getenv ("GDA_PROVIDERS_ROOT_DIR");
 		if (from_dir)
 			prov_list = load_providers_from_dir (from_dir, TRUE);
-		else
-			prov_list = load_providers_from_dir (LIBGDA_PLUGINDIR, FALSE);
+		else {
+			gchar *str;
+			str = gda_gbr_get_file_path (GDA_LIB_DIR, LIBGDA_ABI_NAME, "providers", NULL);
+			prov_list = load_providers_from_dir (str, FALSE);
+			g_free (str);
+		}
 	}
 
 	return prov_list;
@@ -2016,8 +2117,12 @@ gda_config_get_data_source_model (void)
 gboolean
 gda_config_can_modify_global_config (void)
 {
+	gboolean retval;
+	GDA_CONFIG_LOCK ();
 	get_config_client ();
-	return can_modify_global_conf;
+	retval = can_modify_global_conf;
+	GDA_CONFIG_UNLOCK ();
+	return retval;
 }
 
 /**
@@ -2052,8 +2157,11 @@ gda_config_save_data_source (const gchar *name,
 	g_return_val_if_fail (name != NULL, FALSE);
 	g_return_val_if_fail (provider != NULL, FALSE);
 
-	if (is_global && !can_modify_global_conf)
+	GDA_CONFIG_LOCK ();
+	if (is_global && !can_modify_global_conf) {
+		GDA_CONFIG_UNLOCK ();
 		return FALSE;
+	}
 
 	/* delay write and nofity */
 	lock_write_notify = TRUE;
@@ -2119,7 +2227,8 @@ gda_config_save_data_source (const gchar *name,
 	/* actual write and nofity */
 	lock_write_notify = FALSE;
 	write_config_file ();
-	do_notify (NULL);	
+	do_notify (NULL);
+	GDA_CONFIG_UNLOCK ();
 
 	return TRUE;
 }
