@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -87,7 +87,7 @@ struct _GdaProviderInfo {
 
 #define GDA_TYPE_PROVIDER_INFO (gda_provider_info_get_type ())
 
-GType            gda_provider_info_get_type      (void);
+GType            gda_provider_info_get_type      (void) G_GNUC_CONST;
 GdaProviderInfo* gda_provider_info_copy          (GdaProviderInfo *src);
 void             gda_provider_info_free          (GdaProviderInfo *provider_info);
 
@@ -117,8 +117,7 @@ struct _GdaDataSourceInfo {
 
 #define GDA_TYPE_DATA_SOURCE_INFO (gda_data_source_info_get_type ())
 
-GType              gda_data_source_info_get_type    (void);
-GdaDataSourceInfo *gda_data_source_info_new         ();
+GType              gda_data_source_info_get_type    (void) G_GNUC_CONST;
 GdaDataSourceInfo *gda_data_source_info_copy        (GdaDataSourceInfo *src);
 gboolean           gda_data_source_info_equal       (GdaDataSourceInfo *info1, GdaDataSourceInfo *info2);
 GdaDataSourceInfo *gda_config_find_data_source      (const gchar *name);
