@@ -336,7 +336,7 @@ gda_data_model_get_n_rows (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), -1);
 
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_rows)
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_rows) 
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_rows) (model);
 	else 
 		return -1;
@@ -523,7 +523,9 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * Retrieves the data stored in the given position (identified by
  * the @col and @row parameters) on a data model.
  *
- * This is the main function for accessing data in a model.
+ * This is the main function for accessing data in a model which allow random access to its data.
+ * To access data in a data model using a cursor, use a #GdaDataModelIter object, obtained using
+ * gda_data_model_create_iter().
  *
  * Note that the returned #GValue must not be modified directly (unexpected behaviours may
  * occur if you do so). If you want to
@@ -941,7 +943,7 @@ gda_data_model_append_row (GdaDataModel *model, GError **error)
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_INSERT)) {
 		g_set_error (error, 0, 0,
 			     _("Model does not allow row insertion"));
-		return FALSE;
+		return -1;
 	}
 
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_append_row) 
@@ -1670,11 +1672,12 @@ gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node, GEr
  * Returns: TRUE if no error occurred.
  */
 gboolean
-gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, gboolean overwrite, GHashTable *cols_trans, GError **error)
+gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, 
+				  gboolean overwrite, GHashTable *cols_trans, GError **error)
 {
 	GdaDataModelIter *from_iter;
 	gboolean retval = TRUE;
-	gint to_nb_cols, to_nb_rows, to_row = -1;;
+	gint to_nb_cols, to_nb_rows=-1, to_row = -1;
 	gint from_nb_cols;
 	GSList *copy_params = NULL;
 	gint i;
@@ -1832,7 +1835,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, gboolean
 
 		if (retval) {
 			if (to_row >= to_nb_rows)
-				/* we have finished modifying the existng rows */
+				/* we have finished modifying the existing rows */
 				to_row = -1;
 
 			if (to_row >= 0) {
@@ -1940,7 +1943,8 @@ gda_data_model_import_from_file (GdaDataModel *model,
 	return retval;
 }
 
-static gchar *real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes);
+static gchar *real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
+						  gboolean dump_rows, gboolean dump_title, gboolean null_as_empty);
 
 /**
  * gda_data_model_dump
@@ -1948,20 +1952,41 @@ static gchar *real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean
  * @to_stream: where to dump the data model
  *
  * Dumps a textual representation of the @model to the @to_stream stream
+ *
+ * The following environment variables can affect the resulting output:
+ * <itemizedlist>
+ *   <listitem><para>GDA_DATA_MODEL_DUMP_ROW_NUMBERS: if set, the first coulumn of the output will contain row numbers</para></listitem>
+ *   <listitem><para>GDA_DATA_MODEL_DUMP_ATTRIBUTES: if set, also dump the data model's columns' types and value's attributes</para></listitem>
+*   <listitem><para>GDA_DATA_MODEL_DUMP_TITLE: if set, also dump the data model's title</para></listitem>
+*   <listitem><para>GDA_DATA_MODEL_DUMP_NULL_AS_EMPTY: if set, replace the 'NULL' string with an empty string for NULL values </para></listitem>
+ * </itemizedlist>
  */
 void
 gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 {
 	gchar *str;
+	gboolean dump_attrs = FALSE;
+	gboolean dump_rows = FALSE;
+	gboolean dump_title = FALSE;
+	gboolean null_as_empty = FALSE;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	g_return_if_fail (to_stream);
 
-	str = gda_data_model_dump_as_string (model);
+	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) 
+		dump_attrs = TRUE;
+	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS"))
+		dump_rows = TRUE;
+	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) 
+		dump_title = TRUE;
+	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
+		null_as_empty = TRUE;
+
+	str = real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty);
 	g_fprintf (to_stream, "%s", str);
 	g_free (str);
-	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) {
-		str= real_gda_data_model_dump_as_string (model, TRUE);
+	if (dump_attrs) {
+		str = real_gda_data_model_dump_as_string (model, TRUE, dump_rows, dump_title, null_as_empty);
 		g_fprintf (to_stream, "%s", str);
 		g_free (str);
 	}
@@ -1973,14 +1998,31 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
  *
  * Dumps a textual representation of the @model into a new string
  *
+ * The following environment variables can affect the resulting output:
+ * <itemizedlist>
+ *   <listitem><para>GDA_DATA_MODEL_DUMP_ROW_NUMBERS: if set, the first coulumn of the output will contain row numbers</para></listitem>
+*   <listitem><para>GDA_DATA_MODEL_DUMP_TITLE: if set, also dump the data model's title</para></listitem>
+*   <listitem><para>GDA_DATA_MODEL_DUMP_NULL_AS_EMPTY: if set, replace the 'NULL' string with an empty string for NULL values </para></listitem>
+ * </itemizedlist>
  * Returns: a new string.
  */
 gchar *
 gda_data_model_dump_as_string (GdaDataModel *model)
 {
+	gboolean dump_rows = FALSE;
+	gboolean dump_title = FALSE;
+	gboolean null_as_empty = FALSE;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	return real_gda_data_model_dump_as_string (model, FALSE);
+	if (getenv ("GDA_DATA_MODEL_DUMP_ROW_NUMBERS"))
+		dump_rows = TRUE;
+	if (getenv ("GDA_DATA_MODEL_DUMP_TITLE")) 
+		dump_title = TRUE;
+	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
+		null_as_empty = TRUE;
+
+	return real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty);
 }
 
 static void
@@ -2006,7 +2048,8 @@ string_get_dimensions (const gchar *string, gint *width, gint *rows)
 }
 
 static gchar *
-real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes)
+real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
+				    gboolean dump_rows, gboolean dump_title, gboolean null_as_empty)
 {
 #define MULTI_LINE_NO_SEPARATOR
 
@@ -2025,6 +2068,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	const GValue *value;
 
 	gint offset = 0;
+	gint col_offset = dump_rows ? 1 : 0;
 
 	string = g_string_new ("");
 
@@ -2035,9 +2079,16 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	/* compute the columns widths: using column titles... */
 	n_cols = gda_data_model_get_n_columns (model);
 	n_rows = gda_data_model_get_n_rows (model);
-	cols_size = g_new0 (gint, n_cols);
-	cols_is_num = g_new0 (gboolean, n_cols);
+	cols_size = g_new0 (gint, n_cols + col_offset);
+	cols_is_num = g_new0 (gboolean, n_cols + col_offset);
 	
+	if (dump_rows) {
+		str = g_strdup_printf ("%d", n_rows);
+		cols_size [0] = MAX (strlen (str), strlen ("#row"));
+		cols_is_num [0] = TRUE;
+		g_free (str);
+	}
+
 	for (i = 0; i < n_cols; i++) {
 		GdaColumn *gdacol;
 		GType coltype;
@@ -2051,9 +2102,9 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 		str = (gchar *) gda_data_model_get_column_title (model, i);
 		if (str)
-			cols_size [i] = g_utf8_strlen (str, -1);
+			cols_size [i + col_offset] = g_utf8_strlen (str, -1);
 		else
-			cols_size [i] = 6; /* for "<none>" */
+			cols_size [i + col_offset] = 6; /* for "<none>" */
 
 		if (! dump_attributes) {
 			gdacol = gda_data_model_describe_column (model, i);
@@ -2069,12 +2120,12 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 			    (coltype == G_TYPE_CHAR) ||
 			    (coltype == G_TYPE_UCHAR) ||
 			    (coltype == G_TYPE_UINT))
-				cols_is_num [i] = TRUE;
+				cols_is_num [i + col_offset] = TRUE;
 			else
-				cols_is_num [i] = FALSE;
+				cols_is_num [i + col_offset] = FALSE;
 		}
 		else
-			cols_is_num [i] = TRUE;
+			cols_is_num [i + col_offset] = TRUE;
 	}
 
 	/* ... and using column data */
@@ -2082,11 +2133,17 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 		for (i = 0; i < n_cols; i++) {
 			if (! dump_attributes) {
 				value = gda_data_model_get_value_at (model, i, j);
-				str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
+				str = NULL;
+				if (null_as_empty) {
+					if (!value || gda_value_is_null (value))
+						str = g_strdup ("");
+				}
+				if (!str)
+					str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
 				if (str) {
 					gint w;
 					string_get_dimensions (str, &w, NULL);
-					cols_size [i] = MAX (cols_size [i], w);
+					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
 					g_free (str);
 				}
 			}
@@ -2096,26 +2153,49 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 				attrs = gda_data_model_get_attributes_at (model, i, j);
 				str = g_strdup_printf ("%u", attrs);
 				string_get_dimensions (str, &w, NULL);
-				cols_size [i] = MAX (cols_size [i], w);
+				cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
 				g_free (str);
 			}
 		}
 	}
 	
-	/* actual dumping of the contents: column titles...*/
+	/* actual dumping of the contents: title */
+	if (dump_title) {
+		const gchar *title;
+		title = gda_object_get_name (GDA_OBJECT (model));
+		if (title) {
+			gint total_width = n_cols -1, i;
+
+			for (i = 0; i < n_cols; i++)
+				total_width += cols_size [i];
+			if (total_width > strlen (title))
+				i = (total_width - strlen (title))/2.;
+			else
+				i = 0;
+			for (; i > 0; i--)
+				g_string_append_c (string, ' ');
+			g_string_append (string, title);
+			g_string_append_c (string, '\n');
+		}
+	}
+
+	/* ...column titles...*/
+	if (dump_rows) 
+		g_string_append_printf (string, "%*s", cols_size [0], "#row");
+
 	for (i = 0; i < n_cols; i++) {
 		gint j, max;
 		str = (gchar *) gda_data_model_get_column_title (model, i);
-		if (i != 0)
+		if (dump_rows || (i != 0))
 			g_string_append_printf (string, "%s", sep_col);
 
 		if (str) {
 			g_string_append_printf (string, "%s", str);
-			max = cols_size [i] - g_utf8_strlen (str, -1);
+			max = cols_size [i + col_offset] - g_utf8_strlen (str, -1);
 		}
 		else {
 			g_string_append (string, "<none>");
-			max = cols_size [i] - 6;
+			max = cols_size [i + col_offset] - 6;
 		}
 		for (j = 0; j < max; j++)
 			g_string_append_c (string, ' ');
@@ -2123,7 +2203,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_string_append_c (string, '\n');
 		
 	/* ... separation line ... */
-	for (i = 0; i < n_cols; i++) {
+	for (i = 0; i < n_cols + col_offset; i++) {
 		if (i != 0)
 			g_string_append_printf (string, "%s", sep_row);
 		for (j = 0; j < cols_size [i]; j++)
@@ -2135,14 +2215,27 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	if (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM) {
 		for (j = 0; j < n_rows; j++) {
 			/* determine height for each column in that row */
-			gint *cols_height = g_new (gint, n_cols);
-			gchar ***cols_str = g_new (gchar **, n_cols);
-			gint k, kmax = 0;
+			gint *cols_height = g_new (gint, n_cols + col_offset);
+			gchar ***cols_str = g_new (gchar **, n_cols + col_offset);
+			gint k, kmax = 1;
+
+			if (dump_rows) {
+				str = g_strdup_printf ("%d", j);
+				cols_str [0] = g_strsplit (str, "\n", -1);
+				cols_height [0] = 1;
+				kmax = 1;
+			}
 
 			for (i = 0; i < n_cols; i++) {
 				if (!dump_attributes) {
 					value = gda_data_model_get_value_at (model, i, j);
-					str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
+					str = NULL;
+					if (null_as_empty) {
+						if (!value || gda_value_is_null (value))
+							str = g_strdup ("");
+					}
+					if (!str)
+						str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
 				}
 				else {
 					GdaValueAttribute attrs;
@@ -2150,19 +2243,19 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 					str = g_strdup_printf ("%u", attrs);
 				}
 				if (str) {
-					cols_str [i] = g_strsplit (str, "\n", -1);
+					cols_str [i + col_offset] = g_strsplit (str, "\n", -1);
 					g_free (str);
-					cols_height [i] = g_strv_length (cols_str [i]);
-					kmax = MAX (kmax, cols_height [i]);
+					cols_height [i + col_offset] = g_strv_length (cols_str [i + col_offset]);
+					kmax = MAX (kmax, cols_height [i + col_offset]);
 				}
 				else {
-					cols_str [i] = NULL;
-					cols_height [i] = 0;
+					cols_str [i + col_offset] = NULL;
+					cols_height [i + col_offset] = 0;
 				}
 			}
 
 			for (k = 0; k < kmax; k++) {
-				for (i = 0; i < n_cols; i++) {
+				for (i = 0; i < n_cols + col_offset; i++) {
 					gboolean align_center = FALSE;
 					if (i != 0) {
 #ifdef MULTI_LINE_NO_SEPARATOR
@@ -2228,6 +2321,12 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 				g_strfreev (cols_str [i]);
 			g_free (cols_str);
 		}
+		if (n_rows > 1)
+			g_string_append_printf (string, _("(%d rows)\n"), n_rows);
+		else if (n_rows == 1)
+			g_string_append_printf (string, _("(1 row)\n"));
+		else
+			g_string_append_printf (string, _("(no row)\n"));
 	}
 	else 
 		g_string_append (string, _("Model does not support random access, not showing data\n"));
