@@ -44,7 +44,7 @@ gda_entity_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaEntityIface),
 			(GBaseInitFunc) gda_entity_iface_init,
