@@ -24,6 +24,11 @@
 
 #include <libgda/gda-dict-private.h>
 #include <libgda/graph/gda-dict-reg-graphs.h>
+#include <libgda/binreloc/gda-binreloc.h>
+
+static GStaticRecMutex gda_mutex = G_STATIC_REC_MUTEX_INIT;
+#define GDA_LOCK() g_static_rec_mutex_lock(&gda_mutex)
+#define GDA_UNLOCK() g_static_rec_mutex_unlock(&gda_mutex)
 
 static GMainLoop *main_loop = NULL;
 
@@ -49,22 +54,28 @@ gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 {
 	static gboolean initialized = FALSE;
 	GType type;
+	gchar *file;
 
 	if (initialized) {
 		gda_log_error (_("Attempt to re-initialize GDA library. ignored."));
 		return;
 	}
 
-	bindtextdomain (GETTEXT_PACKAGE, LIBGDA_LOCALEDIR);
+	file = gda_gbr_get_file_path (GDA_LOCALE_DIR, NULL);
+	bindtextdomain (GETTEXT_PACKAGE, file);
+	g_free (file);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	/* Threading support if possible */
+	if (!getenv ("LIBGDA_NO_THREADS")) {
 #ifdef G_THREADS_ENABLED
 #ifndef G_THREADS_IMPL_NONE
-	if (! g_thread_supported ())
-		g_thread_init (NULL);
+		if (! g_thread_supported ())
+			g_thread_init (NULL);
 #endif
 #endif
+	}
+
 	g_type_init ();
 	g_set_prgname (app_id);
 
@@ -95,39 +106,56 @@ gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 	type = GDA_TYPE_TIMESTAMP;
 	g_assert (type);
 
+	/* binreloc */
+	gda_gbr_init ();
+
 	/* create a default dictionary */
 	default_dict = gda_dict_new ();
 	gda_dict_register_object_type (default_dict, gda_graphs_get_register ());
 
-#define LIBGDA_DICT_DTD_FILE DTDINSTALLDIR"/libgda-dict.dtd"
-	gda_dict_dtd = xmlParseDTD (NULL, (xmlChar*)LIBGDA_DICT_DTD_FILE);
+	/* dictionary DTD */
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-dict.dtd", NULL);
+	gda_dict_dtd = xmlParseDTD (NULL, (xmlChar*)file);
 	if (gda_dict_dtd)
 		gda_dict_dtd->name = xmlStrdup((xmlChar*) "gda_dict");
 	else
-		g_message (_("Could not parse " LIBGDA_DICT_DTD_FILE ": "
-			     "XML dictionaries validation will not be performed (some weird errors may occur)"));
-#define LIBGDA_PARAMLIST_DTD_FILE DTDINSTALLDIR"/libgda-paramlist.dtd"
-	gda_paramlist_dtd = xmlParseDTD (NULL, (xmlChar*)LIBGDA_PARAMLIST_DTD_FILE);
-	if (gda_paramlist_dtd)
-		gda_paramlist_dtd->name = xmlStrdup((xmlChar*) "data-set-spec");
-	else
-		g_message (_("Could not parse " LIBGDA_PARAMLIST_DTD_FILE ": "
-			     "XML data import validation will not be performed (some weird errors may occur)"));
-#define LIBGDA_ARRAY_DTD_FILE DTDINSTALLDIR"/libgda-array.dtd"
-	gda_array_dtd = xmlParseDTD (NULL, (xmlChar*)LIBGDA_ARRAY_DTD_FILE);
+		g_message (_("Could not parse '%s': "
+			     "XML dictionaries validation will not be performed (some weird errors may occur)"),
+			   file);
+	g_free (file);
+
+	/* array DTD */
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-array.dtd", NULL);
+	gda_array_dtd = xmlParseDTD (NULL, (xmlChar*)file);
 	if (gda_array_dtd)
 		gda_array_dtd->name = xmlStrdup((xmlChar*) "gda_array");
 	else
-		g_message (_("Could not parse " LIBGDA_ARRAY_DTD_FILE ": "
-			     "XML data import validation will not be performed (some weird errors may occur)"));
+		g_message (_("Could not parse '%s': "
+			     "XML data import validation will not be performed (some weird errors may occur)"),
+			   file);
+	g_free (file);
 
-#define LIBGDA_SERVER_OP_DTD_FILE DTDINSTALLDIR"/libgda-server-operation.dtd"
-	gda_server_op_dtd = xmlParseDTD (NULL, (xmlChar*)LIBGDA_SERVER_OP_DTD_FILE);
+	/* paramlist DTD */
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-paramlist.dtd", NULL);
+	gda_paramlist_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+	if (gda_paramlist_dtd)
+		gda_paramlist_dtd->name = xmlStrdup((xmlChar*) "data-set-spec");
+	else
+		g_message (_("Could not parse '%s': "
+			     "XML data import validation will not be performed (some weird errors may occur)"),
+			   file);
+	g_free (file);
+
+	/* server operation DTD */
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-server-operation.dtd", NULL);
+	gda_server_op_dtd = xmlParseDTD (NULL, (xmlChar*)file);
 	if (gda_server_op_dtd)
 		gda_server_op_dtd->name = xmlStrdup((xmlChar*) "serv_op");
 	else
-		g_message (_("Could not parse " LIBGDA_SERVER_OP_DTD_FILE ": "
-			     "Validation for XML files for server operations will not be performed (some weird errors may occur)"));
+		g_message (_("Could not parse '%s': "
+			     "Validation for XML files for server operations will not be performed (some weird errors may occur)"),
+			   file);
+	g_free (file);
 
 	initialized = TRUE;
 }
@@ -180,8 +208,11 @@ idle_cb (gpointer user_data)
 void
 gda_main_run (GdaInitFunc init_func, gpointer user_data)
 {
-	if (main_loop)
+	GDA_LOCK ();
+	if (main_loop) {
+		GDA_UNLOCK ();
 		return;
+	}
 
 	if (init_func) {
 		InitCbData *cb_data;
@@ -195,6 +226,7 @@ gda_main_run (GdaInitFunc init_func, gpointer user_data)
 
 	main_loop = g_main_loop_new (g_main_context_default (), FALSE);
 	g_main_loop_run (main_loop);
+	GDA_UNLOCK ();
 }
 
 /**
@@ -205,10 +237,12 @@ gda_main_run (GdaInitFunc init_func, gpointer user_data)
 void
 gda_main_quit (void)
 {
+	GDA_LOCK ();
 	g_main_loop_quit (main_loop);
 	g_main_loop_unref (main_loop);
 
 	main_loop = NULL;
+	GDA_UNLOCK ();
 }
 
 /*
