@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -126,7 +126,7 @@ gda_query_condition_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaQueryConditionClass),
 			(GBaseInitFunc) NULL,
@@ -1625,14 +1625,18 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 		break;
 	case GDA_QUERY_CONDITION_LEAF_EQUAL:
 		link = "=";
-		if (GDA_IS_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]) &&
+		if (!(options & (GDA_RENDERER_PARAMS_AS_COLON | GDA_RENDERER_PARAMS_AS_DOLLAR |
+				 GDA_RENDERER_PARAMS_AS_QMARK)) &&
+		    GDA_IS_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]) &&
 		    context && 
 		    gda_query_field_value_is_value_null (GDA_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]), context))
-				link = " IS ";
+			link = " IS ";
 		break;
 	case GDA_QUERY_CONDITION_LEAF_DIFF:
 		link = "!=";
-		if (GDA_IS_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]) &&
+		if (!(options & (GDA_RENDERER_PARAMS_AS_COLON | GDA_RENDERER_PARAMS_AS_DOLLAR |
+				 GDA_RENDERER_PARAMS_AS_QMARK)) &&
+		    GDA_IS_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]) &&
 		    context && 
 		    gda_query_field_value_is_value_null (GDA_QUERY_FIELD_VALUE (ops[GDA_QUERY_CONDITION_OP_RIGHT]), context))
 			link = " IS NOT ";
