@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -254,8 +254,8 @@ GType
 gda_row_get_type (void)
 {
 	static GType type = 0;
-	
-	if (!type) {
+
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaRowClass),
 			(GBaseInitFunc) NULL,
@@ -269,7 +269,7 @@ gda_row_get_type (void)
 		};
 		type = g_type_register_static (PARENT_TYPE, "GdaRow", &info, 0);
 	}
-	
+
 	return type;
 }
 
@@ -346,7 +346,7 @@ gda_row_copy (GdaRow *row)
  * @values: a list of #GValue's.
  *
  * Creates a #GdaRow from a list of #GValue's.  These GValue's are
- * value-copied and the user is still responsible for freeing them with g_object_unref().
+ * value-copied and the user are still responsible for freeing them.
  *
  * See the gda_row_new() function's documentation for more information about the @model attribute
  *
@@ -491,7 +491,7 @@ gda_row_set_id (GdaRow *row, const gchar *id)
  * Gets a pointer to a #GValue stored in a #GdaRow.
  *
  * This is a pointer to the internal array of values. Don't try to free
- * or modify it.
+ * or modify it!
  *
  * Returns: a pointer to the #GValue in the position @num of @row.
  */
