@@ -251,7 +251,7 @@ gda_server_operation_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -587,7 +587,7 @@ gda_server_operation_set_property (GObject *object,
 					old_dtd = doc->intSubset;
 					doc->intSubset = gda_server_op_dtd;
 				}
-
+#ifndef G_OS_WIN32
 				if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
 					gchar *str;
 					
@@ -609,6 +609,7 @@ gda_server_operation_set_property (GObject *object,
 					xmlDoValidityCheckingDefaultValue = xmlcheck;
 					return;
 				}
+#endif
 				
 				xmlDoValidityCheckingDefaultValue = xmlcheck;
 				g_free (validc);
@@ -749,25 +750,17 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 
 		/* don't care about entries for the wrong locale */
 		this_lang = xmlGetProp(node, (xmlChar*)"lang");
-		if (this_lang) {
-			if (strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
-				xmlFree (this_lang);
-				node = node->next;
-				continue;
-			}
-
-			xmlFree(this_lang);
-		}
-
-		id = xmlGetProp (node, BAD_CAST "id");
-		if (!id) {
+		if (this_lang && strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
+			g_free (this_lang);
 			node = node->next;
 			continue;
 		}
 
+		id = xmlGetProp (node, BAD_CAST "id");
 		complete_path = g_strdup_printf ("%s/%s", root ? root : "", id);
 		path_name = g_strdup ((gchar*)id);
-		xmlFree (id);
+		if (id)
+			xmlFree (id);
 
 		old_opnode = node_find (op, complete_path);
 		if (old_opnode) {
@@ -924,6 +917,9 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
  * gda_server_operation_new
  * @xml_file: a file which has the specifications for the GdaServerOperation object to create
  *
+ * IMPORTANT NOTE: Using this funtion is not the recommended way of creating a #GdaServerOperation object, the
+ * correct way is to use gda_server_provider_create_operation(); this method is reserved for internal implementation.
+ *
  * Creates a new #GdaServerOperation object from the @xml_file specifications
  *
  * The @xml_file must respect the DTD described in the "libgda-server-operation.dtd" file: its top
@@ -1036,7 +1032,6 @@ gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_fo
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, extension +1))
 						column = NULL;
-					g_free(colid);
 				}
 			}
 			g_free (extension);
@@ -1133,6 +1128,8 @@ gda_server_operation_save_data_to_xml (GdaServerOperation *op, GError **error)
 	g_return_val_if_fail (op->priv, NULL);
 
 	topnode = xmlNewNode (NULL, BAD_CAST "serv_op_data");
+	xmlSetProp (topnode, BAD_CAST "type", 
+		    BAD_CAST gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (op)));
 
 	list = op->priv->topnodes;
 	while (list) {
@@ -1342,7 +1339,13 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 					break;
 				case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
 					gda_data_model_array_clear (GDA_DATA_MODEL_ARRAY (opnode->d.model));
-					if (! gda_data_model_add_data_from_xml_node (opnode->d.model, cur->children, error))
+					if (!cur->children) {
+						g_set_error (error, 0, 0,
+							     _("Missing required settings"));
+						allok = FALSE;
+					}
+					else if (! gda_data_model_add_data_from_xml_node (opnode->d.model, 
+											  cur->children, error))
 						allok = FALSE;
 					break;
 				case GDA_SERVER_OPERATION_NODE_PARAM: {
@@ -1830,7 +1833,6 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 					value = gda_data_model_get_value_at (node_info->model, 
 									     gda_column_get_position (node_info->column), 
 									     row);
-				g_free(extension);
 			}
 			g_free (str);
 		}		
@@ -1848,10 +1850,14 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
  * @path_format: a complete path to a node (starting with "/")
  * @...: arguments to use with @path_format to make a complete path
  *
- * Set the value for the node at the path formed using @path_format and ... the rules are the same as
- * for g_strdup_printf())
+ * Set the value for the node at the path formed using @path_format and @... the rules are the same as
+ * for g_strdup_printf()). 
  *
- * Here are the corner cases:
+ * Note that trying to set a value for a path which is not used by the current
+ * provider (such as "/TABLE_OPTIONS_P/TABLE_ENGINE" for a PostgreSQL connection), will <emphasis>not</emphasis> generate
+ * any error; this allows one to set all the possible parameters and use the same code for several providers.
+ *
+ * Here are the possible formats of @path_format:
  * <itemizedlist>
  *  <listitem><para>If the path corresponds to a #GdaParameter, then the parameter is set to @value</para></listitem>
  *  <listitem><para>If the path corresponds to a sequence item like for example "/SEQUENCE_NAME/5/NAME" for
@@ -1863,7 +1869,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
  *           the value to the corresponding in the 6th item of the sequence</para></listitem>
  *     </itemizedlist>
  *  </para></listitem>
- *  <listitem><para>If the path corresponds to a #GdaDataModel, like for example "/ARRAY/@COLUMN/5" for the value at the
+ *  <listitem><para>If the path corresponds to a #GdaDataModel, like for example "/ARRAY/@@COLUMN/5" for the value at the
  *     6th row of the "COLUMN" column of the "ARRAY" data model, then:
  *     <itemizedlist>
  *        <listitem><para>if the data model already contains 6 or more rows, then the value is just set</para></listitem>
@@ -1960,7 +1966,6 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, colname +1))
 						column = NULL;
-					g_free(colid);
 				}
 				if (column) {
 					gchar *ptr;
@@ -2011,14 +2016,13 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 
 	g_free (extension);
 	g_free (colname);
-	g_free (path);
 	return allok;
 }
 
 /**
  * gda_server_operation_is_valid
  * @op: a #GdaServerOperation widget
- * @xml_file: an XML specification file (see gda_server_operation_new())
+ * @xml_file: a XML specification file (see gda_server_operation_new())
  * @error: a place to store an error, or %NULL
  *
  * Tells if all the required values in @op have been defined.
