@@ -108,7 +108,7 @@ gda_transaction_status_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaTransactionStatusClass),
