@@ -58,7 +58,7 @@ gda_handler_boolean_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerBooleanClass),
 			(GBaseInitFunc) NULL,
