@@ -57,7 +57,7 @@ gda_handler_numerical_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerNumericalClass),
 			(GBaseInitFunc) NULL,
