@@ -84,7 +84,7 @@ gda_handler_time_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerTimeClass),
 			(GBaseInitFunc) NULL,
