@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2004 - 2005 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef SQL_TREE_H
@@ -30,6 +30,7 @@ GdaDelimiterStatement   *gda_delimiter_statement_build  (GdaDelimiterStatementTy
 GdaDelimiterExpr        *gda_delimiter_expr_build       (gchar *str, GList *pspec_list);
 
 /* param spec */
+GList                   *gda_delimiter_param_spec_build_simple (gchar *content);
 GdaDelimiterParamSpec   *gda_delimiter_param_spec_build (GdaDelimiterParamSpecType type, char *content);
 
 #endif
