@@ -52,7 +52,8 @@
      L_PDESCR = 268,
      L_PNULLOK = 269,
      L_UNSPECVAL = 270,
-     L_SC = 271
+     L_SIMPLE_PARAM = 271,
+     L_SC = 272
    };
 #endif
 /* Tokens.  */
@@ -69,7 +70,8 @@
 #define L_PDESCR 268
 #define L_PNULLOK 269
 #define L_UNSPECVAL 270
-#define L_SC 271
+#define L_SIMPLE_PARAM 271
+#define L_SC 272
 
 
 
@@ -84,7 +86,7 @@ GList           *list;
 GdaDelimiterParamSpec     *ps;
 }
 /* Line 1489 of yacc.c.  */
-#line 88 "parser.h"
+#line 90 "parser.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
