@@ -68,6 +68,6 @@ plugin_create_provider (void)
 	GdaServerProvider *prov;
 
         prov = gda_firebird_provider_new ();
-        g_object_set_data (prov, "GDA_PROVIDER_DIR", module_path);
+        g_object_set_data (G_OBJECT (prov), "GDA_PROVIDER_DIR", module_path);
         return prov;
 }
