@@ -51,7 +51,7 @@ struct _GdaMsqlProviderClass {
 typedef struct _GdaMsqlProvider GdaMsqlProvider;
 typedef struct _GdaMsqlProviderClass GdaMsqlProviderClass;
 
-GType gda_msql_provider_get_type(void);
+GType gda_msql_provider_get_type(void) G_GNUC_CONST;
 GdaServerProvider *gda_msql_provider_new(void);
 
 G_END_DECLS
