@@ -1,5 +1,5 @@
 /* GDA Sybase provider
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *         Mike Wingert <wingert.3@postbox.acs.ohio-state.edu>
@@ -115,6 +115,7 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
 	provider_class->string_to_value = NULL;
 	provider_class->get_def_dbms_type = NULL;
 
+	provider_class->create_connection = NULL;
 	provider_class->open_connection = gda_sybase_provider_open_connection;
 	provider_class->close_connection = gda_sybase_provider_close_connection;
 	provider_class->get_database = gda_sybase_provider_get_database;
@@ -126,6 +127,7 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
         provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_sybase_provider_execute_command;
+	provider_class->execute_query = NULL;
 	provider_class->get_last_insert_id = NULL;
 
 	provider_class->begin_transaction = gda_sybase_provider_begin_transaction;
@@ -134,11 +136,8 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
 	provider_class->add_savepoint = NULL;
 	provider_class->rollback_savepoint = NULL;
 	provider_class->delete_savepoint = NULL;
-	
-	provider_class->create_blob = NULL;
-	provider_class->fetch_blob = NULL;
 
-	setlocale(LC_ALL, "C");
+	setlocale (LC_ALL, "C");
 }
 
 static void
@@ -163,7 +162,7 @@ gda_sybase_provider_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static GTypeInfo info = {
 			sizeof (GdaSybaseProviderClass),
 			(GBaseInitFunc) NULL,
@@ -627,7 +626,7 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
                                          const gchar   *sql)
 {
 	GdaSybaseConnectionData *scnc;
-	GdaConnectionEvent                *error;
+	GdaConnectionEvent      *error;
 	gchar                   **arr;
 	GdaSybaseRecordset      *srecset = NULL;	
 	CS_RETCODE              ret;
@@ -650,6 +649,7 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
 	if (arr) {
 		gint n = 0;
 		while (arr[n]) {
+			GdaConnectionEvent *event;
 			GdaDataModel *recset;
 
 			scnc->ret = ct_cmd_alloc (scnc->connection, &scnc->cmd);
@@ -660,6 +660,10 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
 				return NULL;
 			}
 
+			event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+			gda_connection_event_set_description (event, arr[n]);
+			gda_connection_add_event (cnc, event);
+
 			// execute _single_ sql query (one stmt)
 			scnc->ret = ct_command (scnc->cmd, 
 						CS_LANG_CMD,
@@ -766,7 +770,6 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
 						gda_connection_event_set_description (error, _("An error occurred when attempting to test if there is a server message for resultset"));
 						gda_connection_event_set_code (error, -1);
 						gda_connection_event_set_source (error, "gda-sybase");
-						gda_connection_event_set_sqlstate (error, _("Not available"));
 						gda_connection_add_event (cnc, error);		
 						return NULL;
 					}			
@@ -841,7 +844,7 @@ static gboolean
 gda_sybase_execute_cmd (GdaConnection *cnc, const gchar *sql)
 {
 	GdaSybaseConnectionData *scnc;
-	GdaConnectionEvent *error;
+	GdaConnectionEvent *error, *event;
 	gboolean ret = TRUE;
 	CS_INT res_type;
 
@@ -867,6 +870,10 @@ gda_sybase_execute_cmd (GdaConnection *cnc, const gchar *sql)
 		return FALSE;
 	}
 
+	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+	gda_connection_event_set_description (event, sql);
+	gda_connection_add_event (cnc, event);	
+
 	scnc->ret = ct_command (scnc->cmd, CS_LANG_CMD, (CS_CHAR *) sql, 
 				CS_NULLTERM, CS_UNUSED);
 	if (scnc->ret != CS_SUCCEED) {
